// BMP085_output
// by Filipe Vieira
// Simple example of library usage with almost every BMP085 and lib features being used.

#include <Wire.h>
#include <BMP085.h>
#include <LiquidCrystal_I2C.h>
#include <OneWire.h>                        //OneWire Bibliothek einbinden
#include <DallasTemperature.h>              //DallasTemperatureBibliothek einbinden

#define ONE_WIRE_BUS 2                      //Data ist an Pin 2 des Arduinos angeschlossen

OneWire oneWire(ONE_WIRE_BUS);              //Start des OneWire Bus
DallasTemperature sensors(&oneWire);        //Dallas Temperature referenzieren

LiquidCrystal_I2C lcd(0x27,16,2);  // set the LCD address to 0x27 for a 16 chars and 2 line display. ACHTUNG: Adresse kann auch 0x3F sein !!!


// Anschlüsse:
// GND - GND
// VCC - 5V
// SDA - ANALOG Pin 4
// SCL - ANALOG pin 5



BMP085 dps = BMP085();
long Temperature = 0, Pressure = 0, Altitude = 0;


// =======================
// ======== SETUP ========
// =======================

void setup(void)
  {
   Serial.begin(9600);
   Wire.begin();
   sensors.begin();                           // Sensor Start
   delay(1000);
	
   dps.init();   
  
   showall();
  
   Serial.println("Registers dump");
   Serial.println("==========================================================");
   dumpRegisters();
   Serial.println("Calibration data");
   Serial.println("==========================================================");
   dps.dumpCalData();
  
   delay(1000);
  
   lcd.begin();                      // initialize the lcd

   // Print a message to the LCD.
   lcd.backlight();
   lcd.setCursor(0,0);
   lcd.print("T =");
   lcd.setCursor(0,1);
   lcd.print("p =");
   lcd.setCursor(13,0);
   lcd.print("C");
   lcd.setCursor(13,1);
   lcd.print("Pa");  
  }
 

// ===============================
// ======== HAUPTSCHLEIFE ========
// ===============================

void loop(void)
   {
    sensors.requestTemperatures();               // Temperaturen Anfragen 
    Serial.print("Temperatur: "); 
    Serial.println(sensors.getTempCByIndex(0));  // "byIndex(0)" spricht den ersten Sensor an  
    
    dps.getPressure(&Pressure);
          
    /*
    dps.getTemperature(&Temperature); 
    Serial.print("  Temperature:");
    Serial.print(Temperature);
    dps.getAltitude(&Altitude);
    Serial.print("    Alt(m):");
    Serial.println(Altitude/100);
    */
  
    Serial.print("Pressure(Pa):");
    Serial.println(Pressure);
  
    // Ausgabe auf dem LCD-Schirm
    // ==========================

    lcd.setCursor(4,0);
    lcd.print("      ");
    lcd.setCursor(4,0);
    lcd.print(sensors.getTempCByIndex(0));
    
    lcd.setCursor(4,1);
    lcd.print("      ");
    lcd.setCursor(4,1);
    lcd.print(Pressure);
  
    delay(200);
   }



void showall(void)
   { 
    Serial.println("Current BMP085 settings");
    Serial.println("==========================================================");
    Serial.print("device address                  = 0x");
    Serial.println(dps.getDevAddr(), HEX);
    Serial.print("Mode                            = ");
    switch (dps.getMode())
       {
        case MODE_ULTRA_LOW_POWER: 
           Serial.println("MODE_ULTRA_LOW_POWER");
        break;
        case MODE_STANDARD: 
           Serial.println("MODE_STANDARD");
        break;    
        case MODE_HIGHRES: 
           Serial.println("MODE_HIGHRES");
        break;    
        case MODE_ULTRA_HIGHRES:     
           Serial.println("MODE_ULTRA_HIGHRES");
        break; 
       }  
   }


void dumpRegisters()
   {
    byte ValidRegisterAddr[]={0xAA,0xAB,0xAC,0xAD,0xAE,0xAF,0xB0,0xB1,0xB2,0xB3,0xB4,0xB5,0xB6,0xB7,0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0xBE,0xBF,0xF6,0xF7,0xF8,0xF9}; 
    byte _b, i, totregisters = sizeof(ValidRegisterAddr);
    
    Serial.println("---dump start---");
    Serial.println("Register address|Register data");
    Serial.println("Reg.address(hex,dec) Reg.data(bin,hex,dec)");
  
    for (i=0;i<totregisters;i++)
       {    
        Serial.print("0x");
        Serial.print(ValidRegisterAddr[i], HEX);
        Serial.print(",");
        Serial.print(ValidRegisterAddr[i], DEC);
        Serial.print(",");
    
        dps.readmem(ValidRegisterAddr[i], 1, &_b);
    
        Serial.print("b");
        print_bits(_b);
        Serial.print(",0x");
        Serial.print(_b,HEX);
        Serial.print(",");
        Serial.println(_b,DEC);  
       }
    Serial.println("---dump end---");
   }

void print_bits(byte val)
   {
    int i;
  
    for(i=7; i>=0; i--) 
       Serial.print(val >> i & 1, BIN);
   }

/* void print_unit16(uint16_t val){
  int i;
  for(i=15; i>=0; i--) 
    Serial.print(val >> i & 1, BIN);
} 
*/
